@@ -1,24 +1,22 @@
 rcbalance <-
-function(distance.structure, fb.list = NULL, treated.info = NULL, control.info = NULL, target.group = NULL,  k = 1, penalty = 3){
+function(distance.structure, fb.list = NULL, treated.info = NULL, control.info = NULL, exclude.treated = FALSE, target.group = NULL,  k = 1, penalty = 3){
 	#set up treated-control portion of network
 	
-	if(class(distance.structure) %in% c('matrix', 'InfinitySparseMatrix', 'BlockedInfinitySparseMatrix')){
-#	if(inherits(distance.structure, 'matrix')){
-		match.network <- dist2net.matrix(distance.structure,k)
-	}else{
-		match.network <- dist2net(distance.structure,k)
-	}
+
 	####### FINE BALANCE SETUP ######
 	if(!is.null(fb.list)){
 
 		#sanitize input
 		stopifnot(k > 0)
+		stopifnot(penalty > 1)
 		stopifnot(!is.null(treated.info) && !is.null(control.info))
 		stopifnot(ncol(treated.info) == ncol(control.info))
 		stopifnot(all(colnames(treated.info) == colnames(control.info)))
 		#make sure all variables named in fb.list have corresponding columns in the info matrices
 		stopifnot(all(unlist(fb.list) %in% colnames(treated.info)))
 		
+		#exclude.treated is incompatible with k > 1 and with target distributions other than default treated distribution
+		stopifnot(!(exclude.treated && (!is.null(target.group) || k > 1)))
 		
 		if(is.null(target.group)){
 			#unless otherwise specified, target group for fine balance is the treated group
@@ -44,28 +42,48 @@ function(distance.structure, fb.list = NULL, treated.info = NULL, control.info =
 		if(length(fb.list) > 1){
 			for(i in c(1:(length(fb.list)-1)))	stopifnot(all(fb.list[[i]] %in% fb.list[[i+1]]))	
 		}
-	
-
-		for(my.layer in fb.list){
-			interact.factor <- apply(all.subj.info[,match(my.layer, colnames(all.subj.info)), drop = FALSE],1, function(x) paste(x, collapse ='.'))
-			match.network <- add.layer(match.network, interact.factor)
-			#print(paste('After adding',paste(my.layer, collapse = ''),'number of fb layers is',ncol(match.network$fb.structure)))
-		}		
 	}
 	
 
 	##### RUN MATCH #####
+	
+		if(class(distance.structure) %in% c('matrix', 'InfinitySparseMatrix', 'BlockedInfinitySparseMatrix')){
+#	if(inherits(distance.structure, 'matrix')){
+		match.network <- dist2net.matrix(distance.structure,k, exclude.treated = exclude.treated)
+	}else{
+		match.network <- dist2net(distance.structure,k, exclude.treated = exclude.treated)
+	}
+	
+	#add fine balance layers if they are provided
+	if(!is.null(fb.list)){
+		for(my.layer in fb.list){
+				interact.factor <- apply(all.subj.info[,match(my.layer, colnames(all.subj.info)), drop = FALSE],1, function(x) paste(x, collapse ='.'))
+				match.network <- add.layer(match.network, interact.factor)
+		}		
+	}
 		
 	match.network <- penalty.update(match.network, newtheta = penalty) 	
 
+
+	#EDIT: put things in error message to suggest excluding treated if exclude.treated = FALSE?
+
 	if(any(is.na(as.integer(match.network$cost)))){
-		print('Integer overflow in penalty vector!  Run with a lower penalty value.')
+		print('Integer overflow in penalty vector!  Run with a lower penalty value or fewer levels of fine balance.')
 		stop()
 	}
 	o <- callrelax(match.network)	
-	if(o$feasible == 0){	
-		print('Match is infeasible or penalties are too large for RELAX to process!')	
-		stop()
+	if(o$feasible == 0){
+		stub.message <- 'Match is infeasible or penalties are too large for RELAX to process! Consider reducing penalty'
+		if(k > 1){	
+			#print()
+			stop(paste(stub.message, 'or reducing k.'))
+		}
+		if(!exclude.treated){
+			#print()
+			stop(paste(stub.message, 'or setting exclude.treated = TRUE.'))
+		}
+		#print()
+		stop(paste(stub.message, '.', sep =''))
 	}
 	
 	
@@ -74,6 +92,9 @@ function(distance.structure, fb.list = NULL, treated.info = NULL, control.info =
 	#make a |T| x k matrix with rownames equal to index of treated unit and indices of its matched controls stored in each row
 	x <- o$x[1:match.network$tcarcs]	
 	match.df <- data.frame('treat' = as.factor(match.network$startn[1:match.network$tcarcs]), 'x' = x, 'control' = match.network$endn[1:match.network$tcarcs])
+	matched.or.not <- daply(match.df, .(match.df$treat), function(treat.edges) c(treat.edges$treat[1], sum(treat.edges$x)))
+	match.df <- match.df[-which(match.df$treat %in% matched.or.not[which(matched.or.not[,2] == 0),1]),]
+	match.df$treat <- as.factor(as.character(match.df$treat))
 	matches <- daply(match.df, .(match.df$treat), function(treat.edges) treat.edges$control[treat.edges$x == 1])
 
 	#make a contingency table for each fine balance factor 
@@ -81,8 +102,8 @@ function(distance.structure, fb.list = NULL, treated.info = NULL, control.info =
 		fb.tables <- NULL
 	}else{
 		#variables for matched subjects only
-		matched.info <- rbind(treated.info, control.info[as.vector(matches) - sum(match.network$z),])
-		treatment.status <- c(rep(1, nrow(treated.info)), rep(0, k*nrow(treated.info)))
+		matched.info <- rbind(treated.info[as.numeric(names(matches)),], control.info[as.vector(matches) - sum(match.network$z),])
+		treatment.status <- c(rep(1, nrow(as.matrix(matches))), rep(0, k*nrow(as.matrix(matches))))
 		#for each fine balance level k, make a vector of nu_k values for the matched subjects	
 		interact.factors.matched = llply(fb.list, function(my.layer) as.factor(apply(matched.info[,match(my.layer, colnames(matched.info)), drop = FALSE],1, function(x) paste(x, collapse ='.'))))
 		fb.tables <- llply(interact.factors.matched, function(inter.fact) table(inter.fact, treatment.status))	
@@ -91,6 +112,3 @@ function(distance.structure, fb.list = NULL, treated.info = NULL, control.info =
 	#need to decrement match indices to ensure controls are numbered 1:nc again
 	return(list('matches' = matrix(matches - sum(match.network$z), ncol =k, dimnames = list(names(matches),1:k)), 'fb.tables' = fb.tables))
 }
-
-#Both these examples are pretty good!
-#next step - maybe try the InfinitySparseMatrix?  Try using the matrix code I wrote at COR. Check it tomorrow morning?
\ No newline at end of file
